# pylint:disable=too-many-boolean-expressions
from typing import Set, Dict
from collections import defaultdict
import logging

import ailment
import cle

from .optimization_pass import OptimizationPass, OptimizationPassStage


_l = logging.getLogger(name=__name__)


def s2u(s, bits):
    if s > 0:
        return s
    return (1 << bits) + s


class WinStackCanarySimplifier(OptimizationPass):
    """
    Removes stack canary checks from decompilation results for Windows PE files.

    we need to run this pass before performing any full-function simplification. Otherwise the effects of
    _security_cookie will be propagated.
    """

    ARCHES = [
        "X86",
        "AMD64",
    ]
    PLATFORMS = ["windows"]
    STAGE = OptimizationPassStage.AFTER_SINGLE_BLOCK_SIMPLIFICATION
    NAME = "Simplify stack canaries in Windows PE files"
    DESCRIPTION = __doc__.strip()

    def __init__(self, func, **kwargs):
        super().__init__(func, **kwargs)
        self._security_cookie_addr = None
        if isinstance(self.project.loader.main_object, cle.PE):
            self._security_cookie_addr = self.project.loader.main_object.load_config.get("SecurityCookie", None)

        self.analyze()

    def _check(self):
        if self._security_cookie_addr is None:
            return False, None

        # Check the first block and see if there is any statement reading data from _security_cookie
        init_stmt = self._find_canary_init_stmt()

        return init_stmt is not None, {"init_stmt": init_stmt}

    def _analyze(self, cache=None):
        init_stmt = None
        if cache is not None:
            init_stmt = cache.get("init_stmt", None)

        if init_stmt is None:
            init_stmt = self._find_canary_init_stmt()

        if init_stmt is None:
            return

        # Look for the statement that loads back canary value from the stack
        first_block, canary_init_stmt_idx = init_stmt
        canary_init_stmt = first_block.statements[canary_init_stmt_idx]
        # where is the stack canary stored?
        if not isinstance(canary_init_stmt.addr, ailment.Expr.StackBaseOffset):
            _l.debug(
                "Unsupported canary storing location %s. Expects an ailment.Expr.StackBaseOffset.",
                canary_init_stmt.addr,
            )
            return

        store_offset = canary_init_stmt.addr.offset
        if not isinstance(store_offset, int):
            _l.debug("Unsupported canary storing offset %s. Expects an int.", store_offset)

        # The function should have at least one end point calling _security_check_cookie
        # note that (at least for now) we rely on FLIRT to identify the _security_check_cookie function inside the
        # binary.
        # TODO: Add function matching logic to this simplifier

        # Find all nodes with 0 out-degrees
        all_endpoint_addrs = [node.addr for node in self._func.graph.nodes() if self._func.graph.out_degree(node) == 0]

        # Before node duplication, each pair of canary-check-success and canary-check-failure nodes have a common
        # predecessor.
        # map endpoint addrs to their common predecessors
        pred_addr_to_endpoint_addrs: Dict[int, Set[int]] = defaultdict(set)
        for node_addr in all_endpoint_addrs:
            preds = self._func.graph.predecessors(self._func.get_node(node_addr))
            for pred in preds:
                pred_addr_to_endpoint_addrs[pred.addr].add(node_addr)

        found_endpoints = False
        for pred_addr in pred_addr_to_endpoint_addrs:
            # the predecessor should call _security_check_cookie
            endpoint_preds = list(self._get_blocks(pred_addr))
            if self._find_stmt_calling_security_check_cookie(endpoint_preds[0]) is None:
                _l.debug("The predecessor does not call _security_check_cookie().")
                continue

            nodes_to_process = []
            for pred in endpoint_preds:
                check_call_stmt_idx = self._find_stmt_calling_security_check_cookie(pred)
                if check_call_stmt_idx is None:
                    _l.debug("Cannot find the statement calling _security_check_cookie() in the predecessor.")
                    continue

                # TODO: Support x86
                canary_storing_stmt_idx = self._find_amd64_canary_storing_stmt(pred, store_offset)
                if canary_storing_stmt_idx is None:
                    _l.debug("Cannot find the canary check statement in the predecessor.")
                    continue

                return_addr_storing_stmt_idx = self._find_return_addr_storing_stmt(pred)
                if return_addr_storing_stmt_idx is None:
                    _l.debug("Cannot find the return address storing statement in the predecessor.")
                    continue

                nodes_to_process.append(
                    (pred, check_call_stmt_idx, canary_storing_stmt_idx, return_addr_storing_stmt_idx)
                )

            # Now patch this function.
            for pred, check_call_stmt_idx, canary_storing_stmt_idx, return_addr_storing_stmt_idx in nodes_to_process:
                # Patch the pred so that it jumps to the one that is not stack_chk_fail_caller
                stmts = []
                for stmt_idx, stmt in enumerate(pred.statements):
                    if stmt_idx in {check_call_stmt_idx, canary_storing_stmt_idx, return_addr_storing_stmt_idx}:
                        continue
                    stmts.append(stmt)
                pred_copy = pred.copy(statements=stmts)

                self._update_block(pred, pred_copy)

                found_endpoints = True

        if found_endpoints:
            # Remove the statement that loads the stack canary from fs
            first_block_copy = first_block.copy()
            first_block_copy.statements.pop(canary_init_stmt_idx)
            self._update_block(first_block, first_block_copy)

    def _find_canary_init_stmt(self):
        first_block = self._get_block(self._func.addr)
        if first_block is None:
            return None

        for idx, stmt in enumerate(first_block.statements):
            if (
                isinstance(stmt, ailment.Stmt.Store)
                and isinstance(stmt.addr, ailment.Expr.StackBaseOffset)
                and isinstance(stmt.data, ailment.Expr.BinaryOp)
                and stmt.data.op == "Xor"
                and isinstance(stmt.data.operands[1], ailment.Expr.StackBaseOffset)
                and isinstance(stmt.data.operands[0], ailment.Expr.Load)
                and isinstance(stmt.data.operands[0].addr, ailment.Expr.Const)
            ):
                # Check addr: must be __security_cookie
                load_addr = stmt.data.operands[0].addr.value
                if load_addr == self._security_cookie_addr:
                    return first_block, idx

        return None

    @staticmethod
    def _find_amd64_canary_storing_stmt(block, canary_value_stack_offset):
        for idx, stmt in enumerate(block.statements):
            if (
                isinstance(stmt, ailment.Stmt.Assignment)
                and isinstance(stmt.dst, ailment.Expr.Register)
                and stmt.dst.reg_name == "rcx"
            ):
                if isinstance(stmt.src, ailment.Expr.BinaryOp) and stmt.src.op == "Xor":
                    op0, op1 = stmt.src.operands
                    if (
                        isinstance(op0, ailment.Expr.Load)
                        and isinstance(op0.addr, ailment.Expr.StackBaseOffset)
                        and op0.addr.offset == canary_value_stack_offset
                    ):
                        if isinstance(op1, ailment.Expr.StackBaseOffset):
                            # found it
                            return idx

        return None

    @staticmethod
    def _find_return_addr_storing_stmt(block):
        for idx, stmt in enumerate(block.statements):
            if (
                isinstance(stmt, ailment.Stmt.Store)
                and isinstance(stmt.addr, ailment.Expr.StackBaseOffset)
                and isinstance(stmt.data, ailment.Expr.Const)
                and stmt.data.value == block.addr + block.original_size
            ):
                return idx
        return None

    def _find_stmt_calling_security_check_cookie(self, node):
        for idx, stmt in enumerate(node.statements):
            if isinstance(stmt, ailment.Stmt.Call) and isinstance(stmt.target, ailment.Expr.Const):
                const_target = stmt.target.value
                if const_target in self.kb.functions:
                    func = self.kb.functions.function(addr=const_target)
                    if func.name == "_security_check_cookie":
                        return idx

        return None



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("d3d12.dll")
prototypes = \
    {
        #
        'D3D12SerializeRootSignature': SimTypeFunction([SimTypePointer(SimStruct({"NumParameters": SimTypeInt(signed=False, label="UInt32"), "pParameters": SimTypePointer(SimStruct({"ParameterType": SimTypeInt(signed=False, label="D3D12_ROOT_PARAMETER_TYPE"), "Anonymous": SimUnion({"DescriptorTable": SimStruct({"NumDescriptorRanges": SimTypeInt(signed=False, label="UInt32"), "pDescriptorRanges": SimTypePointer(SimStruct({"RangeType": SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_RANGE_TYPE"), "NumDescriptors": SimTypeInt(signed=False, label="UInt32"), "BaseShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "OffsetInDescriptorsFromTableStart": SimTypeInt(signed=False, label="UInt32")}, name="D3D12_DESCRIPTOR_RANGE", pack=False, align=None), offset=0)}, name="D3D12_ROOT_DESCRIPTOR_TABLE", pack=False, align=None), "Constants": SimStruct({"ShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "Num32BitValues": SimTypeInt(signed=False, label="UInt32")}, name="D3D12_ROOT_CONSTANTS", pack=False, align=None), "Descriptor": SimStruct({"ShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32")}, name="D3D12_ROOT_DESCRIPTOR", pack=False, align=None)}, name="<anon>", label="None"), "ShaderVisibility": SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")}, name="D3D12_ROOT_PARAMETER", pack=False, align=None), offset=0), "NumStaticSamplers": SimTypeInt(signed=False, label="UInt32"), "pStaticSamplers": SimTypePointer(SimStruct({"Filter": SimTypeInt(signed=False, label="D3D12_FILTER"), "AddressU": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"), "AddressV": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"), "AddressW": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"), "MipLODBias": SimTypeFloat(size=32), "MaxAnisotropy": SimTypeInt(signed=False, label="UInt32"), "ComparisonFunc": SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC"), "BorderColor": SimTypeInt(signed=False, label="D3D12_STATIC_BORDER_COLOR"), "MinLOD": SimTypeFloat(size=32), "MaxLOD": SimTypeFloat(size=32), "ShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "ShaderVisibility": SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")}, name="D3D12_STATIC_SAMPLER_DESC", pack=False, align=None), offset=0), "Flags": SimTypeInt(signed=False, label="D3D12_ROOT_SIGNATURE_FLAGS")}, name="D3D12_ROOT_SIGNATURE_DESC", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="D3D_ROOT_SIGNATURE_VERSION"), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pRootSignature", "Version", "ppBlob", "ppErrorBlob"]),
        #
        'D3D12CreateRootSignatureDeserializer': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSrcData", "SrcDataSizeInBytes", "pRootSignatureDeserializerInterface", "ppRootSignatureDeserializer"]),
        #
        'D3D12SerializeVersionedRootSignature': SimTypeFunction([SimTypePointer(SimStruct({"Version": SimTypeInt(signed=False, label="D3D_ROOT_SIGNATURE_VERSION"), "Anonymous": SimUnion({"Desc_1_0": SimStruct({"NumParameters": SimTypeInt(signed=False, label="UInt32"), "pParameters": SimTypePointer(SimStruct({"ParameterType": SimTypeInt(signed=False, label="D3D12_ROOT_PARAMETER_TYPE"), "Anonymous": SimUnion({"DescriptorTable": SimStruct({"NumDescriptorRanges": SimTypeInt(signed=False, label="UInt32"), "pDescriptorRanges": SimTypePointer(SimStruct({"RangeType": SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_RANGE_TYPE"), "NumDescriptors": SimTypeInt(signed=False, label="UInt32"), "BaseShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "OffsetInDescriptorsFromTableStart": SimTypeInt(signed=False, label="UInt32")}, name="D3D12_DESCRIPTOR_RANGE", pack=False, align=None), offset=0)}, name="D3D12_ROOT_DESCRIPTOR_TABLE", pack=False, align=None), "Constants": SimStruct({"ShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "Num32BitValues": SimTypeInt(signed=False, label="UInt32")}, name="D3D12_ROOT_CONSTANTS", pack=False, align=None), "Descriptor": SimStruct({"ShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32")}, name="D3D12_ROOT_DESCRIPTOR", pack=False, align=None)}, name="<anon>", label="None"), "ShaderVisibility": SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")}, name="D3D12_ROOT_PARAMETER", pack=False, align=None), offset=0), "NumStaticSamplers": SimTypeInt(signed=False, label="UInt32"), "pStaticSamplers": SimTypePointer(SimStruct({"Filter": SimTypeInt(signed=False, label="D3D12_FILTER"), "AddressU": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"), "AddressV": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"), "AddressW": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"), "MipLODBias": SimTypeFloat(size=32), "MaxAnisotropy": SimTypeInt(signed=False, label="UInt32"), "ComparisonFunc": SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC"), "BorderColor": SimTypeInt(signed=False, label="D3D12_STATIC_BORDER_COLOR"), "MinLOD": SimTypeFloat(size=32), "MaxLOD": SimTypeFloat(size=32), "ShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "ShaderVisibility": SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")}, name="D3D12_STATIC_SAMPLER_DESC", pack=False, align=None), offset=0), "Flags": SimTypeInt(signed=False, label="D3D12_ROOT_SIGNATURE_FLAGS")}, name="D3D12_ROOT_SIGNATURE_DESC", pack=False, align=None), "Desc_1_1": SimStruct({"NumParameters": SimTypeInt(signed=False, label="UInt32"), "pParameters": SimTypePointer(SimStruct({"ParameterType": SimTypeInt(signed=False, label="D3D12_ROOT_PARAMETER_TYPE"), "Anonymous": SimUnion({"DescriptorTable": SimStruct({"NumDescriptorRanges": SimTypeInt(signed=False, label="UInt32"), "pDescriptorRanges": SimTypePointer(SimStruct({"RangeType": SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_RANGE_TYPE"), "NumDescriptors": SimTypeInt(signed=False, label="UInt32"), "BaseShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_RANGE_FLAGS"), "OffsetInDescriptorsFromTableStart": SimTypeInt(signed=False, label="UInt32")}, name="D3D12_DESCRIPTOR_RANGE1", pack=False, align=None), offset=0)}, name="D3D12_ROOT_DESCRIPTOR_TABLE1", pack=False, align=None), "Constants": SimStruct({"ShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "Num32BitValues": SimTypeInt(signed=False, label="UInt32")}, name="D3D12_ROOT_CONSTANTS", pack=False, align=None), "Descriptor": SimStruct({"ShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="D3D12_ROOT_DESCRIPTOR_FLAGS")}, name="D3D12_ROOT_DESCRIPTOR1", pack=False, align=None)}, name="<anon>", label="None"), "ShaderVisibility": SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")}, name="D3D12_ROOT_PARAMETER1", pack=False, align=None), offset=0), "NumStaticSamplers": SimTypeInt(signed=False, label="UInt32"), "pStaticSamplers": SimTypePointer(SimStruct({"Filter": SimTypeInt(signed=False, label="D3D12_FILTER"), "AddressU": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"), "AddressV": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"), "AddressW": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"), "MipLODBias": SimTypeFloat(size=32), "MaxAnisotropy": SimTypeInt(signed=False, label="UInt32"), "ComparisonFunc": SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC"), "BorderColor": SimTypeInt(signed=False, label="D3D12_STATIC_BORDER_COLOR"), "MinLOD": SimTypeFloat(size=32), "MaxLOD": SimTypeFloat(size=32), "ShaderRegister": SimTypeInt(signed=False, label="UInt32"), "RegisterSpace": SimTypeInt(signed=False, label="UInt32"), "ShaderVisibility": SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")}, name="D3D12_STATIC_SAMPLER_DESC", pack=False, align=None), offset=0), "Flags": SimTypeInt(signed=False, label="D3D12_ROOT_SIGNATURE_FLAGS")}, name="D3D12_ROOT_SIGNATURE_DESC1", pack=False, align=None)}, name="<anon>", label="None")}, name="D3D12_VERSIONED_ROOT_SIGNATURE_DESC", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pRootSignature", "ppBlob", "ppErrorBlob"]),
        #
        'D3D12CreateVersionedRootSignatureDeserializer': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSrcData", "SrcDataSizeInBytes", "pRootSignatureDeserializerInterface", "ppRootSignatureDeserializer"]),
        #
        'D3D12CreateDevice': SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAdapter", "MinimumFeatureLevel", "riid", "ppDevice"]),
        #
        'D3D12GetDebugInterface': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["riid", "ppvDebug"]),
        #
        'D3D12EnableExperimentalFeatures': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), label="LPArray", offset=0), SimTypePointer(SimTypeBottom(label="Void"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["NumFeatures", "pIIDs", "pConfigurationStructs", "pConfigurationStructSizes"]),
    }

lib.set_prototypes(prototypes)



#!/usr/bin/env python3

import os
import time

import angr
import claripy

bvs = claripy.BVS("foo", 8)

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../../../binaries")
state = angr.Project(
    os.path.join(test_location, "tests", "x86_64", "fauxware"), main_opts={"base_addr": 0x400000}, auto_load_libs=True
).factory.full_init_state(add_options={angr.options.REVERSE_MEMORY_NAME_MAP})


def cycle(s):
    s = s.copy()
    s.memory.store(0x400000, bvs)
    return s


def main():
    s = cycle(state)
    for _ in range(20000):
        s = cycle(s)


if __name__ == "__main__":
    tstart = time.time()
    main()
    tend = time.time()
    print("Elapsed: %f sec" % (tend - tstart))



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.sim.exec_func"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")

arches = {"mips", "mipsel", "mips64", "x86_64", "ppc", "ppc64"}


class TestSyscallResult(unittest.TestCase):
    @staticmethod
    def run_test_syscalls(arch):
        p = angr.Project(os.path.join(test_location, arch, "test_ioctl"), exclude_sim_procedures_list=["ioctl"])
        p.simos.syscall_library.procedures.pop("ioctl", None)

        s = p.factory.entry_state()

        simgr = p.factory.simulation_manager(thing=s)
        simgr.run()
        assert (
            len(simgr.deadended) == 2
        ), "for these architectures, libc checks if the bit is set. make sure it branches"

    def test_syscalls(self):
        for arch in arches:
            with self.subTest(arch=arch):
                self.run_test_syscalls(arch)


if __name__ == "__main__":
    unittest.main()



import angr


class select(angr.SimProcedure):
    # pylint:disable=arguments-differ

    def run(self, nfds, readfds, writefds, exceptfds, timeout):  # pylint: disable=unused-argument
        try:
            nfds_v = self.state.solver.eval_one(nfds)
            # readfds_v = self.state.solver.eval_one(readfds)
            writefds_v = self.state.solver.eval_one(writefds)
            exceptfds_v = self.state.solver.eval_one(exceptfds)
        except angr.errors.SimSolverError:
            raise angr.errors.SimProcedureArgumentError("Can't handle symbolic select arguments")

        if writefds_v != 0 or exceptfds_v != 0:
            raise angr.errors.SimProcedureError("Can't handle write or exception events in select")

        arch_bits = self.arch.bits
        arch_bytes = self.arch.bytes

        long_array = []
        long_array_size = ((nfds_v - 1) + arch_bits) // arch_bits
        for offset in range(0, long_array_size):
            long = self.state.memory.load(readfds + offset * arch_bytes, arch_bytes, endness=self.arch.memory_endness)
            long_array.append(long)
        for i in range(0, nfds_v - 1):
            # get a bit
            long_pos = i // arch_bits
            bit_offset = i % arch_bits
            bit = long_array[long_pos][bit_offset]

            if bit.symbolic or self.state.solver.eval(bit) == 1:
                # set this bit to symbolic
                long_array[long_pos] = (
                    long_array[long_pos][arch_bits - 1 : bit_offset + 1]
                    .concat(self.state.solver.BVS("fd_state", 1))
                    .concat(long_array[long_pos][bit_offset - 1 :])
                )

        # write things back
        for offset in range(0, long_array_size):
            self.state.memory.store(readfds + offset * arch_bytes, long_array[offset], endness=self.arch.memory_endness)

        retval = self.state.solver.BVV(0, 1).concat(self.state.solver.BVS("select_ret", 31))
        return retval



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import os
import unittest

import angr
import angr.analyses.decompiler

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestClinic(unittest.TestCase):
    def test_smoketest(self):
        binary_path = os.path.join(test_location, "x86_64", "all")
        proj = angr.Project(binary_path, auto_load_libs=False, load_debug_info=True)

        cfg = proj.analyses.CFG(normalize=True)
        main_func = cfg.kb.functions["main"]

        proj.analyses.Clinic(main_func)


if __name__ == "__main__":
    unittest.main()



import angr
import logging

from .VirtualAlloc import convert_prot, deconvert_prot

l = logging.getLogger(name=__name__)


class VirtualProtect(angr.SimProcedure):
    def run(self, lpAddress, dwSize, flNewProtect, lpfOldProtect):
        l.debug("VirtualProtect(%s, %s, %s, %s)", lpAddress, dwSize, flNewProtect, lpfOldProtect)
        addrs = self.state.solver.eval_upto(lpAddress, 2)
        if len(addrs) != 1:
            raise angr.errors.SimValueError("VirtualProtect can't handle symbolic lpAddress")
        addr = addrs[0]

        size = self.state.solver.max_int(dwSize)
        if dwSize.symbolic and size > self.state.libc.max_variable_size:
            l.warning(
                "symbolic VirtuaProtect dwSize %s has maximum %#x, greater than state.libc.max_variable_size %#x",
                dwSize,
                size,
                self.state.libc.max_variable_size,
            )
            size = self.state.libc.max_variable_size

        prots = self.state.solver.eval_upto(flNewProtect, 2)
        if len(prots) != 1:
            raise angr.errors.SimValueError("VirtualProtect can't handle symbolic flNewProtect")
        prot = prots[0]

        try:
            if not self.state.solver.is_false(self.state.memory.permissions(lpfOldProtect) & 2 == 0):
                l.debug("...failed, bad lpfOldProtect (write-perm)")
                return 0
        except angr.errors.SimMemoryError:
            l.debug("...failed, bad lpfOldProtect (write-miss)")
            return 0

        page_start = addr & ~0xFFF
        page_end = (addr + size - 1) & ~0xFFF
        first_prot = None
        try:
            for page in range(page_start, page_end + 0x1000, 0x1000):
                old_prot = self.state.memory.permissions(page)
                if first_prot is None:
                    first_prot = self.state.solver.eval(old_prot)
        except angr.errors.SimMemoryError:
            l.debug("...failed, bad address")
            return 0

        angr_prot = convert_prot(prot)

        # we're good! make the changes.
        for page in range(page_start, page_end + 0x1000, 0x1000):
            self.state.memory.permissions(page, angr_prot)

        self.state.mem[lpfOldProtect].dword = deconvert_prot(first_prot)
        return 1



import logging

from claripy import StringV, StrLen

from . import JNISimProcedure
from ...engines.soot.values import SimSootValue_StringRef

l = logging.getLogger("angr.procedures.java_jni.string_operations")

# pylint: disable=arguments-differ,unused-argument

#
# GetStringUTFChars
#


class GetStringUTFChars(JNISimProcedure):
    return_ty = "reference"

    def run(self, ptr_env, str_ref_, ptr_isCopy):
        # get string value
        str_ref = self.state.jni_references.lookup(str_ref_)
        str_val = self.state.javavm_memory.load(str_ref)

        # store string in native memory
        addr = self._store_string_in_native_memory(str_val)

        # if isCopy is not null, store JNI_TRUE at that address
        if self.state.solver.eval(ptr_isCopy != 0):
            self._store_in_native_memory(data=self.JNI_TRUE, data_type="boolean", addr=ptr_isCopy)

        return addr


#
# ReleaseStringUTFChars
#


class ReleaseStringUTFChars(JNISimProcedure):
    return_ty = "void"

    def run(self, ptr_env, str_ref_, native_buf):
        # FIXME delete this, when claripy fully supports string solving
        # All changes made from the native code to a symbolic string are not
        # reflected in the claripy.String representation (used for Java strings)
        # => when the native code releases a string, the symbolic string gets
        #    replaced with a concretized version that includes all additional
        #    constraints added in the native execution
        str_ref = self.state.jni_references.lookup(str_ref_)
        str_val = self.state.javavm_memory.load(str_ref)
        if self.state.solver.symbolic(str_val):
            str_native = self._load_string_from_native_memory(native_buf)
            self.state.javavm_memory.store(str_ref, StringV(str_native))


#
# NewStringUTF
#


class NewStringUTF(JNISimProcedure):
    return_ty = "reference"

    def run(self, ptr_env, ptr_str_bytes):
        # load string from native memory
        str_val = self._load_string_from_native_memory(ptr_str_bytes)

        # create java string and return the reference
        str_ref = SimSootValue_StringRef(self.state.javavm_memory.get_new_uuid())
        self.state.javavm_memory.store(str_ref, StringV(str_val))
        return self.state.jni_references.create_new_reference(str_ref)


#
# GetStringUTFLength
#


class GetStringUTFLength(JNISimProcedure):
    return_ty = "int"

    def run(self, ptr_env, str_ref_):
        str_ref = self.state.jni_references.lookup(str_ref_)
        str_val = self.state.javavm_memory.load(str_ref)
        return StrLen(str_val, 32)



#!/usr/bin/env python3
__package__ = __package__ or "tests.factory"  # pylint:disable=redefined-builtin

import os
import unittest

import claripy
import archinfo

import angr
from angr.sim_type import SimTypePointer, SimTypeFunction, SimTypeChar, SimTypeInt, parse_defns
from angr.errors import AngrCallableMultistateError


from ..common import bin_location, slow_test


test_location = os.path.join(bin_location, "tests")

addresses_fauxware = {
    "armel": 0x8524,
    "armhf": 0x104C9,  # addr+1 to force thumb
    "i386": 0x8048524,
    "mips": 0x400710,
    "mipsel": 0x4006D0,
    "ppc": 0x1000054C,
    "ppc64": 0x10000698,
    "x86_64": 0x400664,
}

addresses_manysum = {
    "armel": 0x1041C,
    "armhf": 0x103BD,
    "i386": 0x80483D8,
    "mips": 0x400704,
    "mipsel": 0x400704,
    "ppc": 0x10000418,
    "ppc64": 0x10000500,
    "x86_64": 0x4004CA,
}

type_cache = None


class TestCallable(unittest.TestCase):
    def run_fauxware(self, arch):
        addr = addresses_fauxware[arch]
        p = angr.Project(os.path.join(test_location, arch, "fauxware"))
        charstar = SimTypePointer(SimTypeChar())
        prototype = SimTypeFunction((charstar, charstar), SimTypeInt(False))
        authenticate = p.factory.callable(
            addr, toc=0x10018E80 if arch == "ppc64" else None, concrete_only=True, prototype=prototype
        )
        assert authenticate("asdf", "SOSNEAKY").concrete_value == 1
        self.assertRaises(AngrCallableMultistateError, authenticate, "asdf", "NOSNEAKY")

    def run_callable_c_fauxware(self, arch):
        addr = addresses_fauxware[arch]
        p = angr.Project(os.path.join(test_location, arch, "fauxware"))
        authenticate = p.factory.callable(
            addr, toc=0x10018E80 if arch == "ppc64" else None, concrete_only=True, prototype="int f(char*, char*)"
        )
        retval = authenticate.call_c('("asdf", "SOSNEAKY")')
        assert retval.concrete_value == 1

    def run_manysum(self, arch):
        addr = addresses_manysum[arch]
        p = angr.Project(os.path.join(test_location, arch, "manysum"))
        inttype = SimTypeInt()
        prototype = SimTypeFunction([inttype] * 11, inttype)
        sumlots = p.factory.callable(addr, prototype=prototype)
        result = sumlots(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11)
        assert not result.symbolic
        assert result.concrete_value == sum(range(12))

    def run_callable_c_manysum(self, arch):
        addr = addresses_manysum[arch]
        p = angr.Project(os.path.join(test_location, arch, "manysum"))
        sumlots = p.factory.callable(addr, prototype="int f(int, int, int, int, int, int, int, int, int, int, int)")
        result = sumlots.call_c("(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11)")
        assert not result.symbolic
        assert result.concrete_value == sum(range(12))

    def run_manyfloatsum(self, arch):
        global type_cache
        if type_cache is None:
            with open(os.path.join(bin_location, "tests_src", "manyfloatsum.c")) as fp:
                type_cache = parse_defns(fp.read())

        p = angr.Project(os.path.join(test_location, arch, "manyfloatsum"))
        for function in (
            "sum_floats",
            "sum_combo",
            "sum_segregated",
            "sum_doubles",
            "sum_combo_doubles",
            "sum_segregated_doubles",
        ):
            args = list(range(len(type_cache[function].args)))
            answer = float(sum(args))
            addr = p.loader.main_object.get_symbol(function).rebased_addr
            my_callable = p.factory.callable(addr, prototype=type_cache[function])
            result = my_callable(*args)
            assert not result.symbolic
            result_concrete = result.args[0]
            assert answer == result_concrete

    @slow_test
    def run_manyfloatsum_symbolic(self, arch):
        global type_cache
        if type_cache is None:
            with open(os.path.join(bin_location, "tests_src", "manyfloatsum.c")) as fp:
                type_cache = parse_defns(fp.read())

        p = angr.Project(os.path.join(test_location, arch, "manyfloatsum"))
        function = "sum_doubles"
        args = [claripy.FPS("arg_%d" % i, claripy.FSORT_DOUBLE) for i in range(len(type_cache[function].args))]
        addr = p.loader.main_object.get_symbol(function).rebased_addr
        my_callable = p.factory.callable(addr, prototype=type_cache[function])
        result = my_callable(*args)
        assert result.symbolic

        s = claripy.Solver(timeout=15 * 60 * 1000)
        for arg in args:
            s.add(arg > claripy.FPV(1.0, claripy.FSORT_DOUBLE))
        s.add(result == claripy.FPV(27.7, claripy.FSORT_DOUBLE))

        args_conc = s.batch_eval(args, 1)[0]
        assert s.eval(result, 1)[0] == 27.7
        # not almost equal!! totally equal!!! z3 is magic, if kinda slow!!!!!
        for arg_conc in args_conc:
            assert arg_conc > 1.0
        assert sum(args_conc) == 27.7

    def test_fauxware_armel(self):
        self.run_fauxware("armel")

    def test_fauxware_armhf(self):
        self.run_fauxware("armhf")

    def test_fauxware_i386(self):
        self.run_fauxware("i386")

    def test_fauxware_mips(self):
        self.run_fauxware("mips")

    def test_fauxware_mipsel(self):
        self.run_fauxware("mipsel")

    def test_fauxware_ppc(self):
        self.run_fauxware("ppc")

    def test_fauxware_ppc64(self):
        self.run_fauxware("ppc64")

    def test_fauxware_x86_64(self):
        self.run_fauxware("x86_64")

    def test_manysum_armel(self):
        self.run_manysum("armel")

    def test_manysum_armhf(self):
        self.run_manysum("armhf")

    def test_manysum_i386(self):
        self.run_manysum("i386")

    def test_manysum_mips(self):
        self.run_manysum("mips")

    def test_manysum_mipsel(self):
        self.run_manysum("mipsel")

    def test_manysum_ppc(self):
        self.run_manysum("ppc")

    def test_manysum_ppc64(self):
        self.run_manysum("ppc64")

    def test_manysum_x86_64(self):
        self.run_manysum("x86_64")

    def test_manyfloatsum_i386(self):
        self.run_manyfloatsum("i386")

    def test_manyfloatsum_x86_64(self):
        self.run_manyfloatsum("x86_64")

    @slow_test
    def test_manyfloatsum_symbolic_i386(self):
        # doesn't have to be slow but it might be
        # https://github.com/Z3Prover/z3/issues/2584
        self.run_manyfloatsum_symbolic("i386")

    @slow_test
    def test_manyfloatsum_symbolic_x86_64(self):
        # doesn't have to be slow but it might be
        # https://github.com/Z3Prover/z3/issues/2584
        self.run_manyfloatsum_symbolic("x86_64")

    def test_callable_c_fauxware_armel(self):
        self.run_callable_c_fauxware("armel")

    def test_callable_c_fauxware_armhf(self):
        self.run_callable_c_fauxware("armhf")

    def test_callable_c_fauxware_i386(self):
        self.run_callable_c_fauxware("i386")

    def test_callable_c_fauxware_mips(self):
        self.run_callable_c_fauxware("mips")

    def test_callable_c_fauxware_mipsel(self):
        self.run_callable_c_fauxware("mipsel")

    def test_callable_c_fauxware_ppc(self):
        self.run_callable_c_fauxware("ppc")

    def test_callable_c_fauxware_ppc64(self):
        self.run_callable_c_fauxware("ppc64")

    def test_callable_c_fauxware_x86_64(self):
        self.run_callable_c_fauxware("x86_64")

    def test_callable_c_manyfloatsum_armel(self):
        self.run_callable_c_manysum("armel")

    def test_callable_c_manyfloatsum_armhf(self):
        self.run_callable_c_manysum("armhf")

    def test_callable_c_manyfloatsum_i386(self):
        self.run_callable_c_manysum("i386")

    def test_callable_c_manyfloatsum_mips(self):
        self.run_callable_c_manysum("mips")

    def test_callable_c_manyfloatsum_mipsel(self):
        self.run_callable_c_manysum("mipsel")

    def test_callable_c_manyfloatsum_ppc(self):
        self.run_callable_c_manysum("ppc")

    def test_callable_c_manyfloatsum_ppc64(self):
        self.run_callable_c_manysum("ppc64")

    def test_callable_c_manyfloatsum_x86_64(self):
        self.run_callable_c_manysum("x86_64")

    def test_setup_callsite(self):
        p = angr.load_shellcode(b"b", arch=archinfo.ArchX86())

        s = p.factory.call_state(
            0, "hello", prototype="void x(char*)", stack_base=0x1234, alloc_base=0x5678, grow_like_stack=False
        )
        assert (s.regs.sp == 0x1234).is_true()
        assert (s.mem[0x1234 + 4].long.resolved == 0x5678).is_true()
        assert (s.memory.load(0x5678, 5) == b"hello").is_true()

        s = p.factory.call_state(0, "hello", prototype="void x(char*)", stack_base=0x1234)
        assert (s.regs.sp == 0x1234).is_true()
        assert (s.mem[0x1234 + 4].long.resolved == 0x1234 + 8).is_true()
        assert (s.memory.load(0x1234 + 8, 5) == b"hello").is_true()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.procedures.libc"  # pylint:disable=redefined-builtin

import os
import subprocess
import sys
import unittest

import angr

from ...common import slow_test, bin_location  # pylint:disable=import-error,wrong-import-position


class TestStrtol(unittest.TestCase):
    # pylint: disable=no-self-use

    @slow_test
    @unittest.skipUnless(sys.platform.startswith("linux"), "linux-only")
    def test_strtol(self, threads=None):
        test_bin = os.path.join(bin_location, "tests", "x86_64", "strtol_test")
        # disabling auto_load_libs increases the execution time of the test case.
        b = angr.Project(test_bin, auto_load_libs=True)

        initial_state = b.factory.entry_state(remove_options={angr.options.LAZY_SOLVES})
        pg = b.factory.simulation_manager(thing=initial_state, threads=threads)

        # find the end of main
        expected_outputs = {
            b"base 8 worked\n",
            b"base +8 worked\n",
            b"0x worked\n",
            b"+0x worked\n",
            b"base +10 worked\n",
            b"base 10 worked\n",
            b"base -8 worked\n",
            b"-0x worked\n",
            b"base -10 worked\n",
            b"Nope\n",
        }
        pg.explore(find=0x400804, num_find=len(expected_outputs))
        assert len(pg.found) == len(expected_outputs)

        # check the outputs
        pipe = subprocess.PIPE
        for f in pg.found:
            test_input = f.posix.dumps(0)
            test_output = f.posix.dumps(1)
            expected_outputs.remove(test_output)

            # check the output works as expected
            with subprocess.Popen(test_bin, stdout=pipe, stderr=pipe, stdin=pipe) as p:
                ret = p.communicate(test_input)[0]
            assert ret == test_output

        # check that all of the outputs were seen
        assert len(expected_outputs) == 0

    def test_strtol_long_string(self):
        # convert a 11-digit long string to a number.
        # there was an off-by-one error before.

        b = angr.load_shellcode(b"\x90\x90", "AMD64")
        state = b.factory.blank_state()
        state.memory.store(0x500000, b"98831114236\x00")

        state.libc.max_strtol_len = 11

        strtol = angr.SIM_LIBRARIES["libc.so.6"].get("strtol", arch=b.arch)
        strtol.state = state.copy()
        ret = strtol.run(0x500000, 0, 0)

        assert strtol.state.satisfiable()
        assert len(strtol.state.solver.eval_upto(ret, 2)) == 1
        assert strtol.state.solver.eval_one(ret) == 98831114236


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import logging
import os
import unittest

import angr

from ..common import bin_location


l = logging.getLogger("angr.tests.test_bindiff")

test_location = os.path.join(bin_location, "tests")


# todo make a better test
# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestBindiff(unittest.TestCase):
    def test_bindiff_x86_64(self):
        binary_path_1 = os.path.join(test_location, "x86_64", "bindiff_a")
        binary_path_2 = os.path.join(test_location, "x86_64", "bindiff_b")
        b = angr.Project(binary_path_1, load_options={"auto_load_libs": False})
        b2 = angr.Project(binary_path_2, load_options={"auto_load_libs": False})
        bindiff = b.analyses.BinDiff(b2)

        identical_functions = bindiff.identical_functions
        differing_functions = bindiff.differing_functions
        unmatched_functions = bindiff.unmatched_functions
        # check identical functions
        assert (0x40064C, 0x40066A) in identical_functions
        # check differing functions
        assert (0x400616, 0x400616) in differing_functions
        # check unmatched functions
        assert len(unmatched_functions[0]) <= 1
        assert len(unmatched_functions[1]) <= 2
        # check for no major regressions
        assert len(identical_functions) > len(differing_functions)
        assert len(differing_functions) < 4

        # check a function diff
        fdiff = bindiff.get_function_diff(0x400616, 0x400616)
        block_matches = {(a.addr, b.addr) for a, b in fdiff.block_matches}
        assert (0x40064A, 0x400668) in block_matches
        assert (0x400616, 0x400616) in block_matches
        assert (0x40061E, 0x40061E) in block_matches


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.factory"  # pylint:disable=redefined-builtin

import os
import unittest

import claripy

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestArgcSym(unittest.TestCase):
    def _verify_results(self, pg, sargc, length=400):
        argcs = pg.mp_found.solver.eval(sargc)
        strs = pg.mp_found.solver.eval(pg.mp_found.memory.load(pg.mp_found.regs.sp, length), cast_to=bytes)

        for a, s in zip(argcs.mp_items, strs.mp_items):
            assert a in (0, 1, 2)
            assert b"Good man" in s if a == 1 else b"Very Good man" if a == 2 else True

    def test_mips(self):
        arger_mips = angr.Project(os.path.join(test_location, "mips", "argc_symbol"), auto_load_libs=False)
        r_addr = [0x400720, 0x40076C, 0x4007BC]

        sargc = claripy.BVS("argc", 32)
        s = arger_mips.factory.entry_state(
            args=[claripy.BVS("arg_0", 40 * 8), claripy.BVS("arg_1", 40 * 8), claripy.BVS("arg_2", 40 * 8)],
            env={"HOME": "/home/angr"},
            argc=sargc,
        )
        pg = arger_mips.factory.simulation_manager(s).explore(find=r_addr, num_find=100)
        self._verify_results(pg, sargc)

    def test_mipsel(self):
        arger_mipsel = angr.Project(os.path.join(test_location, "mipsel", "argc_symbol"), auto_load_libs=False)
        r_addr = [0x400720, 0x40076C, 0x4007BC]

        sargc = claripy.BVS("argc", 32)
        s = arger_mipsel.factory.entry_state(
            args=[claripy.BVS("arg_0", 40 * 8), claripy.BVS("arg_1", 40 * 8), claripy.BVS("arg_2", 40 * 8)],
            env={"HOME": "/home/angr"},
            argc=sargc,
        )
        pg = arger_mipsel.factory.simulation_manager(s).explore(find=r_addr, num_find=100)
        self._verify_results(pg, sargc)

    def test_i386(self):
        arger_i386 = angr.Project(os.path.join(test_location, "i386", "argc_symbol"), auto_load_libs=False)
        r_addr = [0x08048411, 0x08048437, 0x08048460]

        sargc = claripy.BVS("argc", 32)
        s = arger_i386.factory.entry_state(
            args=[claripy.BVS("arg_0", 40 * 8), claripy.BVS("arg_1", 40 * 8), claripy.BVS("arg_2", 40 * 8)],
            env={"HOME": "/home/angr"},
            argc=sargc,
        )
        pg = arger_i386.factory.simulation_manager(s).explore(find=r_addr, num_find=100)
        self._verify_results(pg, sargc)

    def test_amd64(self):
        arger_amd64 = angr.Project(
            os.path.join(test_location, "x86_64", "argc_symbol"), load_options={"auto_load_libs": False}
        )
        r_addr = [0x40051B, 0x400540, 0x400569]

        sargc = claripy.BVS("argc", 64)
        s = arger_amd64.factory.entry_state(
            args=[claripy.BVS("arg_0", 40 * 8), claripy.BVS("arg_1", 40 * 8), claripy.BVS("arg_2", 40 * 8)],
            env={"HOME": "/home/angr"},
            argc=sargc,
        )
        pg = arger_amd64.factory.simulation_manager(s).explore(find=r_addr, num_find=100)
        self._verify_results(pg, sargc, length=800)

    def test_arm(self):
        arger_arm = angr.Project(os.path.join(test_location, "armel", "argc_symbol"), auto_load_libs=False)
        r_addr = [0x00010444, 0x00010478, 0x000104B0]

        sargc = claripy.BVS("argc", 32)
        s = arger_arm.factory.entry_state(
            args=[claripy.BVS("arg_0", 40 * 8), claripy.BVS("arg_1", 40 * 8), claripy.BVS("arg_2", 40 * 8)],
            env={"HOME": "/home/angr"},
            argc=sargc,
        )
        pg = arger_arm.factory.simulation_manager(s).explore(find=r_addr, num_find=100)
        self._verify_results(pg, sargc)

    def test_ppc32(self):
        arger_ppc32 = angr.Project(os.path.join(test_location, "ppc", "argc_symbol"), auto_load_libs=False)
        r_addr = [0x1000043C, 0x10000474, 0x100004B0]

        sargc = claripy.BVS("argc", 32)
        s = arger_ppc32.factory.entry_state(
            args=[claripy.BVS("arg_0", 40 * 8), claripy.BVS("arg_1", 40 * 8), claripy.BVS("arg_2", 40 * 8)],
            env={"HOME": "/home/angr"},
            argc=sargc,
        )
        pg = arger_ppc32.factory.simulation_manager(s).explore(find=r_addr, num_find=100)
        self._verify_results(pg, sargc)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.factory"  # pylint:disable=redefined-builtin

import logging
import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestWindowsArgs(unittest.TestCase):
    def test_i386(self):
        after_puts = 0x40105B
        else_paths = [0x401062, 0x401009]

        p = angr.Project(os.path.join(test_location, "i386", "simple_windows.exe"), auto_load_libs=False)

        s = p.factory.entry_state(args=("simple_windows.exe", "angr_can_windows?", "1497715489"))
        simgr = p.factory.simulation_manager(s)
        simgr.explore(find=after_puts, avoid=else_paths, num_find=10)

        assert len(simgr.avoid) == 0
        assert len(simgr.found) > 0
        for f in simgr.found:
            assert b"ok" in f.posix.dumps(1)


if __name__ == "__main__":
    logging.getLogger("angr.engines").setLevel("INFO")
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.state_plugins"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestSymbolization(unittest.TestCase):
    def test_fauxware_symbolization(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
        sm = p.factory.simulation_manager()

        assert not sm.one_active.regs.rsp.symbolic

        sm.one_active.symbolizer.set_symbolization_for_all_pages()
        sm.one_active.symbolizer.resymbolize()

        # assert sm.one_active.regs.rsp.symbolic
        assert sm.one_active.symbolizer.symbolized_count

        # make sure pointers get symbolized at runtime
        n = sm.one_active.symbolizer.symbolized_count
        sm.run()
        assert not sm.errored
        assert not sm.active
        assert sm.one_deadended.symbolizer.symbolized_count > n


if __name__ == "__main__":
    unittest.main()



import angr


class strdup(angr.SimProcedure):
    # pylint:disable=arguments-differ

    def run(self, s):
        strlen = angr.SIM_PROCEDURES["libc"]["strlen"]
        strncpy = angr.SIM_PROCEDURES["libc"]["strncpy"]
        malloc = angr.SIM_PROCEDURES["libc"]["malloc"]

        src_len = self.inline_call(strlen, s).ret_expr
        new_s = self.inline_call(malloc, src_len + 1).ret_expr

        self.inline_call(strncpy, new_s, s, src_len + 1, src_len=src_len)

        return new_s



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.exploration_techniques"  # pylint:disable=redefined-builtin

import logging
import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")
log = logging.getLogger("angr_tests.test_proxy")


class TestTechBuilder(unittest.TestCase):
    def test_tech_builder(self):
        # pylint:disable=unused-argument
        class Foo:
            @staticmethod
            def setup(*args, **kwargs):
                log.debug("setup() triggered!")

            @staticmethod
            def step_state(*args, **kwargs):
                log.debug("step_state() triggered!")

            @staticmethod
            def step(simgr, stash, *args, **kwargs):
                log.debug("step() triggered!")
                return simgr.step(stash=stash, **kwargs)

            @staticmethod
            def filter(*args, **kwargs):
                log.debug("filter() triggered!")

            @staticmethod
            def complete(*args, **kwargs):
                log.debug("complete() triggered!")
                return True

        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), load_options={"auto_load_libs": False})

        foo = Foo()
        proxy_tech = angr.exploration_techniques.TechniqueBuilder(
            setup=foo.setup,
            step_state=foo.step_state,
            step=foo.step,
            filter=foo.filter,
            complete=foo.complete,
        )

        pg = p.factory.simulation_manager()
        pg.use_technique(proxy_tech)
        pg.run()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.exploration_techniques"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from .test_tracer import tracer_cgc
from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestDrillerCore(unittest.TestCase):
    def test_cgc(self):
        binary = os.path.join(test_location, "cgc", "sc1_0b32aa01_01")
        simgr, tracer = tracer_cgc(binary, "driller_core_cgc", b"AAAA", copy_states=True, follow_unsat=True)
        simgr.use_technique(angr.exploration_techniques.DrillerCore(tracer._trace))
        simgr.run()

        assert "diverted" in simgr.stashes
        assert len(simgr.diverted) == 3

    def test_simprocs(self):
        binary = os.path.join(test_location, "i386", "driller_simproc")
        memcmp = angr.SIM_PROCEDURES["libc"]["memcmp"]()

        simgr, tracer = tracer_cgc(binary, "driller_core_simprocs", b"A" * 128, copy_states=True, follow_unsat=True)
        p = simgr._project
        p.hook(0x8048200, memcmp)

        d = angr.exploration_techniques.DrillerCore(tracer._trace)
        simgr.use_technique(d)

        simgr.run()
        assert "diverted" in simgr.stashes
        assert len(simgr.diverted) > 0


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.knowledge_plugins"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestKbPluginsDwarf(unittest.TestCase):
    def test_kb_plugins_dwarf(self):
        p = angr.Project(
            os.path.join(test_location, "x86_64", "state_merge_0"),
            load_options={
                "load_debug_info": True,
                "auto_load_libs": False,
            },
        )
        assert isinstance(p.kb.variables, angr.knowledge_plugins.VariableManager)
        p.kb.variables.load_from_dwarf()

        ret = p.kb.variables.global_manager.get_global_variables(6295620)
        assert len(ret) == 1

        v = ret.pop()
        assert v.name == "buf"


if __name__ == "__main__":
    unittest.main()



from angr.utils.graph import GraphUtils
from .graph import GraphVisitor


class CallGraphVisitor(GraphVisitor):
    """
    :param networkx.DiGraph callgraph:
    """

    def __init__(self, callgraph):
        super().__init__()
        self.callgraph = callgraph

        self.reset()

    def successors(self, node):
        return list(self.callgraph.successors(node))

    def predecessors(self, node):
        return list(self.callgraph.predecessors(node))

    def sort_nodes(self, nodes=None):
        sorted_nodes = GraphUtils.quasi_topological_sort_nodes(self.callgraph)

        if nodes is not None:
            sorted_nodes = [n for n in sorted_nodes if n in set(nodes)]

        return sorted_nodes



#!/usr/bin/env python3
import unittest

from angr.analyses.cfg_slice_to_sink.transitions import merge_transitions


class TestTrasitions(unittest.TestCase):
    def test_merge_transitions(self):
        t1 = {1: [2, 3]}
        t2 = {2: [4]}

        result = merge_transitions(t1, t2)
        expected_result = {1: [2, 3], 2: [4]}

        self.assertEqual(result, expected_result)

    def test_merging_transitions_with_a_from_node_present_in_both_operands(self):
        t1 = {1: [2, 3]}
        t2 = {1: [4]}

        result = merge_transitions(t1, t2)
        expected_result = {1: [2, 3, 4]}

        self.assertEqual(result, expected_result)


if __name__ == "__main__":
    unittest.main()



# pylint:disable=unused-import
from ..models import DbComment
from ...knowledge_plugins.comments import Comments


class CommentsSerializer:
    """
    Serialize/unserialize comments to/from a database session.
    """

    @staticmethod
    def dump(session, db_kb, comments):
        """

        :param session:
        :param DbKnowledgeBase db_kb:
        :param Comments comments:
        :return:                        None
        """

        for addr, comment in comments.items():
            db_comment = (
                session.query(DbComment)
                .filter_by(
                    kb=db_kb,
                    addr=addr,
                )
                .scalar()
            )
            if db_comment is not None:
                if comment == db_comment.comment:
                    continue
                db_comment.comment = comment
            else:
                db_comment = DbComment(
                    kb=db_kb,
                    addr=addr,
                    comment=comment,
                    type=0,
                )
                session.add(db_comment)

    @staticmethod
    def load(session, db_kb, kb):  # pylint:disable=unused-argument
        """

        :param session:
        :param DbKnowledgeBase db_kb:
        :param KnowledgeBase kb:
        :return:
        """

        db_comments = db_kb.comments
        comments = Comments(kb)

        for db_comment in db_comments:
            comments[db_comment.addr] = db_comment.comment

        return comments



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long

import unittest

from angr import SimState, SimFile, SIM_PROCEDURES


class TestPwrite(unittest.TestCase):
    def test_pwrite(self):
        pwrite = SIM_PROCEDURES["posix"]["pwrite64"]()

        state = SimState(arch="AMD64", mode="symbolic")
        simfile = SimFile("concrete_file", content="hello world!\n")
        state.fs.insert("test", simfile)
        fd = state.posix.open(b"test", 1)

        buf_addr = 0xD0000000
        state.memory.store(buf_addr, b"test!")
        pwrite.execute(state, arguments=[fd, buf_addr, 5, 6])

        simfd = state.posix.get_fd(fd)
        simfd.seek(0)
        res = 0xC0000000
        simfd.read(res, 13)
        data = state.solver.eval(state.mem[res].string.resolved, cast_to=bytes)

        assert data == b"hello test!!\n"

        state.posix.close(fd)


class TestPread(unittest.TestCase):
    def test_pread(self):
        pwrite = SIM_PROCEDURES["posix"]["pread64"]()

        state = SimState(arch="AMD64", mode="symbolic")
        simfile = SimFile("concrete_file", content="hello world!\n")
        state.fs.insert("test", simfile)
        fd = state.posix.open(b"test", 1)

        buf1_addr = 0xD0000000
        buf2_addr = 0xD0001000
        pwrite.execute(state, arguments=[fd, buf1_addr, 6, 6])
        pwrite.execute(state, arguments=[fd, buf2_addr, 5, 0])

        data1 = state.solver.eval(state.mem[buf1_addr].string.resolved, cast_to=bytes)
        data2 = state.solver.eval(state.mem[buf2_addr].string.resolved, cast_to=bytes)

        assert data1 == b"world!"
        assert data2 == b"hello"

        state.posix.close(fd)


if __name__ == "__main__":
    unittest.main()
